/*  Keyboard driver for interpreter. Takes input one line at a time
 *  from stdin , generates a program vector and executes it .Loops
 *  until EOF. Note: Due to an apparent bug in the Venix i/o driver,
 *  EOF (ctrl-d) is ignored unless it is the first character in
 *  a line 
 */

#include <dos.h>
#include <errno.h>
#include <conio.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <signal.h>
#include <at200.h>

#include "idecls.h"
#define  MAIN_PROG	/* Needed for camsys.h */
#include "camsys.h"
#include "proto.h"
#include "mode.h"

#define FKEY		0
#define RIGHT_ARROW	0x4D
#define LEFT_ARROW	0x4B
#define PGUP		0x49
#define PGDN		0x51
#define CTRL_PGUP	0x84
#define CTRL_PGDN	0x76

/* Function prototypes used in this program. */

void _cdecl main ( int, char *[] );
int sabrti ( void );
int sabrtos ( void );

void _cdecl main( int argc, char *argv[] )
{
	register int len;
	register char *ep;
	int next;
	entry_ptr pv[50];
	char line[ MAXLINE+1 ];
	
	abrt = NORMAL;
	signal(SIGINT, sabrti);
	signal(SIGQUIT, sabrtos);
	st_init();
	init_poca( argc, argv );
	
	/* Main processing loop. Accept commands from keyboard or 
	 * script file until terminated by stop command, or program
	 * exception. 
	 */

	while (abrt != OSABORT) 
	{
		abrt = NORMAL;
		delete();		/* Clear symbol table. */
		if ( textmode )
			printf ("*");
		if ( line[0] )		/* Initial command line args */
			len = strlen ( line );
		else if ( ( len = getline( stdin, line, MAXLINE ) ) == EOF) 
		{
			terminate_ki();
		}
		if ( ( len == MAXLINE ) && ( textmode ) )
		{
			printf ("Input line too long.\n");
			continue; 
		}
		if ( len > 0 )
		{
			if ( ( ( ep = iparse( line, pv, &next ) ) != NULL )
				&& ( textmode == TRUE ) )
				printf("%s",ep) ;
			else
			{
				pv[next] = NULL ;
				iexec(pv) ;
			}
		}
		line[0] = '\0';
	}
	terminate_ki();
}


/* set abrt to IABORT on interrupt signal */

int sabrti ( void )
{
	abrt = IABORT; 
	signal (SIGINT, sabrti) ;
	return(0) ;
}

/* set abrt to OSABORT on quit signal */

int sabrtos ( void )
{
	abrt = OSABORT ;
	signal(SIGQUIT, sabrtos);
	return(0);
}

/*  Get a line from file pointer fp and put in s. Returns length
 *  of line or EOF. 
 *  If input stream is 'stdin', then yield control to a chron
 *  function to log temperatures at specified intervals.
 */

int getline ( FILE *fp, char *s, register int lim )
{
int c, i = 0, comment = 0;
	
while ( lim > 0 )
{
   c = 0x7F;  /* c is the value returned from the keyboard. */
             /* Setting it to an unused value is a kludge to keep out of an infinite loop. */
   
    if ( fp != stdin ) {       /* Input from script file. */ 
	if ( ( c = getc( fp ) ) == EOF ) 
		return (EOF);
    }
    else {        /* Input stream is stdin. (console) */
         if ( !kbhit() )
            chron_func();
         else {
            if ( textmode )
	       c = getche();
	    else
	       c = getch();
	 if ( c == 0x1A )     /* Control-Z */
	    return EOF;
	 }
   }
   if ( c != 0x7F ){
      switch( c ){	    		    
         case FKEY:  /* If character is a special function key, get scan code */
            c = getch();
            switch( c ){
               case RIGHT_ARROW:       /* Go to graphics mode */
                  Select_Graphics();
	          textmode = FALSE;
                  break;
               case LEFT_ARROW:        /* Go to text mode */
	          textmode = TRUE;
	          setvideomode( TEXT_MODE );
	          break;
	       }
	    c = 0x7F;
	    return( i );
	    break;      /* End of inner switch */
	    
	 /* If no special function key pressed then it's ASCII */
	 
   case '!':           /* Comment line in script file */
      comment = i;
      break;
   case '\b':           /* Backspace */
      if ( i ){
         s[ i-- ] = '\0';
         lim++;
      }
      break;
   case '\r':
   case '\n':           /* CR or LF */
      s[ i ] = '\0' ;
      if ( cmndlogmode ){
         fprintf( cmndlogfl, "%s\n", s );
         fflush( cmndlogfl );
      }
		/* If input from stdin, send newline. */	
	
         if ( ( fp == stdin ) && ( textmode ) )
            printf( "\n" );
         else if ( textmode ){	/* Display script line */
            printf("%s\n", s );
            
/*            sprintf( omsg, "%s\n", s );
	    disp_omsg( NOERROR );*/
	 }
	 if (comment )		/* Truncate comment portion */
	    s[ comment ] = '\0';
	 return ( i ) ;
      default:                /* Default value should be ASCII character */
         s[ i++ ] = (char)c;
         lim--;
         break;
      }
   }
   }
}
